@@ -74,6 +74,9 @@ class BaseMIL(LightningContainer, LoadingParams, EncoderParams, PoolingParams, C
     save_intermediate_outputs: bool = param.Boolean(
         True, doc="Whether to save intermediate validation outputs during training."
     )
+    stratify_plots_by: Optional[str] = param.String(None,
+                                                    doc="Name of metadata field to stratify output plots"
+                                                    "(PR curve, ROC curve).")
 
     def __init__(self, **kwargs: Any) -> None:
         super().__init__(**kwargs)
@@ -148,6 +151,7 @@ class BaseMIL(LightningContainer, LoadingParams, EncoderParams, PoolingParams, C
             test_plot_options=self.get_test_plot_options(),
             loading_params=create_from_matching_params(self, LoadingParams),
             save_intermediate_outputs=self.save_intermediate_outputs,
+            stratify_plots_by=self.stratify_plots_by
         )
         if self.num_top_slides > 0:
             outputs_handler.tiles_selector = TilesSelector(
@@ -220,6 +224,7 @@ class BaseMIL(LightningContainer, LoadingParams, EncoderParams, PoolingParams, C
                                        analyse_loss=self.analyse_loss)
         deepmil_module.transfer_weights(self.trained_weights_path)
         outputs_handler.set_slides_dataset_for_plots_handlers(self.get_slides_dataset())
+        outputs_handler.set_extra_slides_dataset_for_plots_handlers(self.get_extra_slides_dataset_for_plotting())
         return deepmil_module
 
     def get_data_module(self) -> HistoDataModule:
@@ -228,6 +233,9 @@ class BaseMIL(LightningContainer, LoadingParams, EncoderParams, PoolingParams, C
     def get_slides_dataset(self) -> Optional[SlidesDataset]:
         return None
 
+    def get_extra_slides_dataset_for_plotting(self) -> Optional[SlidesDataset]:
+        return None
+
     def ignore_pl_warnings(self) -> None:
         # Pytorch Lightning prints a warning if the batch size is not consistent across all batches. The way PL infers
         # the batch size is not compatible with our data loaders. It searches for the first item in the batch that is a
